@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#include <config.h>
-
 #include <sys/time.h>
 #include <sys/types.h>
 
@@ -30,16 +28,19 @@
 
 #include <dev/usb/usb.h>
 
+#include "libusb.h"
 #include "libusbi.h"
 
 struct device_priv {
 	char *devname;				/* name of the ugen(4) node */
 	int fd;					/* device file descriptor */
 
-	usb_config_descriptor_t *cdesc;		/* active config descriptor */
+	unsigned char *cdesc;			/* active config descriptor */
+	usb_device_descriptor_t ddesc;		/* usb device descriptor */
 };
 
 struct handle_priv {
+	int pipe[2];				/* for event notification */
 	int endpoints[USB_MAX_ENDPOINTS];
 };
 
@@ -51,25 +52,31 @@ static int obsd_get_device_list(struct libusb_context *,
 static int obsd_open(struct libusb_device_handle *);
 static void obsd_close(struct libusb_device_handle *);
 
+static int obsd_get_device_descriptor(struct libusb_device *, unsigned char *,
+    int *);
 static int obsd_get_active_config_descriptor(struct libusb_device *,
-    void *, size_t);
+    unsigned char *, size_t, int *);
 static int obsd_get_config_descriptor(struct libusb_device *, uint8_t,
-    void *, size_t);
+    unsigned char *, size_t, int *);
 
-static int obsd_get_configuration(struct libusb_device_handle *, uint8_t *);
+static int obsd_get_configuration(struct libusb_device_handle *, int *);
 static int obsd_set_configuration(struct libusb_device_handle *, int);
 
-static int obsd_claim_interface(struct libusb_device_handle *, uint8_t);
-static int obsd_release_interface(struct libusb_device_handle *, uint8_t);
+static int obsd_claim_interface(struct libusb_device_handle *, int);
+static int obsd_release_interface(struct libusb_device_handle *, int);
 
-static int obsd_set_interface_altsetting(struct libusb_device_handle *, uint8_t,
-    uint8_t);
+static int obsd_set_interface_altsetting(struct libusb_device_handle *, int,
+    int);
 static int obsd_clear_halt(struct libusb_device_handle *, unsigned char);
+static int obsd_reset_device(struct libusb_device_handle *);
 static void obsd_destroy_device(struct libusb_device *);
 
 static int obsd_submit_transfer(struct usbi_transfer *);
 static int obsd_cancel_transfer(struct usbi_transfer *);
-static int obsd_handle_transfer_completion(struct usbi_transfer *);
+static void obsd_clear_transfer_priv(struct usbi_transfer *);
+static int obsd_handle_events(struct libusb_context *ctx, struct pollfd *,
+    nfds_t, int);
+static int obsd_clock_gettime(int, struct timespec *);
 
 /*
  * Private functions
@@ -83,32 +90,48 @@ static int _access_endpoint(struct libusb_transfer *);
 static int _bus_open(int);
 
 
-const struct usbi_os_backend usbi_backend = {
-	.name = "Synchronous OpenBSD backend",
-	.get_device_list = obsd_get_device_list,
-	.open = obsd_open,
-	.close = obsd_close,
+const struct usbi_os_backend openbsd_backend = {
+	"Synchronous OpenBSD backend",
+	0,
+	NULL,				/* init() */
+	NULL,				/* exit() */
+	obsd_get_device_list,
+	NULL,				/* hotplug_poll */
+	obsd_open,
+	obsd_close,
 
-	.get_active_config_descriptor = obsd_get_active_config_descriptor,
-	.get_config_descriptor = obsd_get_config_descriptor,
+	obsd_get_device_descriptor,
+	obsd_get_active_config_descriptor,
+	obsd_get_config_descriptor,
+	NULL,				/* get_config_descriptor_by_value() */
 
-	.get_configuration = obsd_get_configuration,
-	.set_configuration = obsd_set_configuration,
+	obsd_get_configuration,
+	obsd_set_configuration,
 
-	.claim_interface = obsd_claim_interface,
-	.release_interface = obsd_release_interface,
+	obsd_claim_interface,
+	obsd_release_interface,
 
-	.set_interface_altsetting = obsd_set_interface_altsetting,
-	.clear_halt = obsd_clear_halt,
-	.destroy_device = obsd_destroy_device,
+	obsd_set_interface_altsetting,
+	obsd_clear_halt,
+	obsd_reset_device,
 
-	.submit_transfer = obsd_submit_transfer,
-	.cancel_transfer = obsd_cancel_transfer,
+	NULL,				/* kernel_driver_active() */
+	NULL,				/* detach_kernel_driver() */
+	NULL,				/* attach_kernel_driver() */
 
-	.handle_transfer_completion = obsd_handle_transfer_completion,
+	obsd_destroy_device,
 
-	.device_priv_size = sizeof(struct device_priv),
-	.device_handle_priv_size = sizeof(struct handle_priv),
+	obsd_submit_transfer,
+	obsd_cancel_transfer,
+	obsd_clear_transfer_priv,
+
+	obsd_handle_events,
+
+	obsd_clock_gettime,
+	sizeof(struct device_priv),
+	sizeof(struct handle_priv),
+	0,				/* transfer_priv_size */
+	0,				/* add_iso_packet_size */
 };
 
 #define DEVPATH	"/dev/"
@@ -129,7 +152,7 @@ obsd_get_device_list(struct libusb_context * ctx,
 	char *udevname;
 	int fd, addr, i, j;
 
-	usbi_dbg(ctx, " ");
+	usbi_dbg("");
 
 	for (i = 0; i < 8; i++) {
 		snprintf(busnode, sizeof(busnode), USBDEV "%d", i);
@@ -173,10 +196,10 @@ obsd_get_device_list(struct libusb_context * ctx,
 				dev->bus_number = di.udi_bus;
 				dev->device_address = di.udi_addr;
 				dev->speed = di.udi_speed;
-				dev->port_number = di.udi_port;
 
-				dpriv = usbi_get_device_priv(dev);
+				dpriv = (struct device_priv *)dev->os_priv;
 				dpriv->fd = -1;
+				dpriv->cdesc = NULL;
 				dpriv->devname = udevname;
 
 				dd.udd_bus = di.udi_bus;
@@ -185,11 +208,7 @@ obsd_get_device_list(struct libusb_context * ctx,
 					libusb_unref_device(dev);
 					continue;
 				}
-
-				static_assert(sizeof(dev->device_descriptor) == sizeof(dd.udd_desc),
-					      "mismatch between libusb and OS device descriptor sizes");
-				memcpy(&dev->device_descriptor, &dd.udd_desc, LIBUSB_DT_DEVICE_SIZE);
-				usbi_localize_device_descriptor(&dev->device_descriptor);
+				dpriv->ddesc = dd.udd_desc;
 
 				if (_cache_active_config_descriptor(dev)) {
 					libusb_unref_device(dev);
@@ -222,59 +241,85 @@ obsd_get_device_list(struct libusb_context * ctx,
 int
 obsd_open(struct libusb_device_handle *handle)
 {
-	struct device_priv *dpriv = usbi_get_device_priv(handle->dev);
+	struct handle_priv *hpriv = (struct handle_priv *)handle->os_priv;
+	struct device_priv *dpriv = (struct device_priv *)handle->dev->os_priv;
 	char devnode[16];
 
 	if (dpriv->devname) {
-		int fd;
 		/*
 		 * Only open ugen(4) attached devices read-write, all
 		 * read-only operations are done through the bus node.
 		 */
 		snprintf(devnode, sizeof(devnode), DEVPATH "%s.00",
 		    dpriv->devname);
-		fd = open(devnode, O_RDWR);
-		if (fd < 0)
+		dpriv->fd = open(devnode, O_RDWR);
+		if (dpriv->fd < 0)
 			return _errno_to_libusb(errno);
-		dpriv->fd = fd;
 
-		usbi_dbg(HANDLE_CTX(handle), "open %s: fd %d", devnode, dpriv->fd);
+		usbi_dbg("open %s: fd %d", devnode, dpriv->fd);
 	}
 
-	return (LIBUSB_SUCCESS);
+	if (pipe(hpriv->pipe) < 0)
+		return _errno_to_libusb(errno);
+
+	return usbi_add_pollfd(HANDLE_CTX(handle), hpriv->pipe[0], POLLIN);
 }
 
 void
 obsd_close(struct libusb_device_handle *handle)
 {
-	struct device_priv *dpriv = usbi_get_device_priv(handle->dev);
+	struct handle_priv *hpriv = (struct handle_priv *)handle->os_priv;
+	struct device_priv *dpriv = (struct device_priv *)handle->dev->os_priv;
 
 	if (dpriv->devname) {
-		usbi_dbg(HANDLE_CTX(handle), "close: fd %d", dpriv->fd);
+		usbi_dbg("close: fd %d", dpriv->fd);
 
 		close(dpriv->fd);
 		dpriv->fd = -1;
 	}
+
+	usbi_remove_pollfd(HANDLE_CTX(handle), hpriv->pipe[0]);
+
+	close(hpriv->pipe[0]);
+	close(hpriv->pipe[1]);
+}
+
+int
+obsd_get_device_descriptor(struct libusb_device *dev, unsigned char *buf,
+    int *host_endian)
+{
+	struct device_priv *dpriv = (struct device_priv *)dev->os_priv;
+
+	usbi_dbg("");
+
+	memcpy(buf, &dpriv->ddesc, DEVICE_DESC_LENGTH);
+
+	*host_endian = 0;
+
+	return (LIBUSB_SUCCESS);
 }
 
 int
 obsd_get_active_config_descriptor(struct libusb_device *dev,
-    void *buf, size_t len)
+    unsigned char *buf, size_t len, int *host_endian)
 {
-	struct device_priv *dpriv = usbi_get_device_priv(dev);
+	struct device_priv *dpriv = (struct device_priv *)dev->os_priv;
+	usb_config_descriptor_t *ucd = (usb_config_descriptor_t *)dpriv->cdesc;
 
-	len = MIN(len, (size_t)UGETW(dpriv->cdesc->wTotalLength));
+	len = MIN(len, UGETW(ucd->wTotalLength));
 
-	usbi_dbg(DEVICE_CTX(dev), "len %zu", len);
+	usbi_dbg("len %d", len);
 
 	memcpy(buf, dpriv->cdesc, len);
 
-	return ((int)len);
+	*host_endian = 0;
+
+	return (len);
 }
 
 int
 obsd_get_config_descriptor(struct libusb_device *dev, uint8_t idx,
-    void *buf, size_t len)
+    unsigned char *buf, size_t len, int *host_endian)
 {
 	struct usb_device_fdesc udf;
 	int fd, err;
@@ -288,7 +333,7 @@ obsd_get_config_descriptor(struct libusb_device *dev, uint8_t idx,
 	udf.udf_size = len;
 	udf.udf_data = buf;
 
-	usbi_dbg(DEVICE_CTX(dev), "index %d, len %zu", udf.udf_config_index, len);
+	usbi_dbg("index %d, len %d", udf.udf_config_index, len);
 
 	if (ioctl(fd, USB_DEVICE_GET_FDESC, &udf) < 0) {
 		err = errno;
@@ -297,17 +342,20 @@ obsd_get_config_descriptor(struct libusb_device *dev, uint8_t idx,
 	}
 	close(fd);
 
-	return ((int)len);
+	*host_endian = 0;
+
+	return (len);
 }
 
 int
-obsd_get_configuration(struct libusb_device_handle *handle, uint8_t *config)
+obsd_get_configuration(struct libusb_device_handle *handle, int *config)
 {
-	struct device_priv *dpriv = usbi_get_device_priv(handle->dev);
+	struct device_priv *dpriv = (struct device_priv *)handle->dev->os_priv;
+	usb_config_descriptor_t *ucd = (usb_config_descriptor_t *)dpriv->cdesc;
 
-	*config = dpriv->cdesc->bConfigurationValue;
+	*config = ucd->bConfigurationValue;
 
-	usbi_dbg(HANDLE_CTX(handle), "bConfigurationValue %u", *config);
+	usbi_dbg("bConfigurationValue %d", *config);
 
 	return (LIBUSB_SUCCESS);
 }
@@ -315,12 +363,12 @@ obsd_get_configuration(struct libusb_device_handle *handle, uint8_t *config)
 int
 obsd_set_configuration(struct libusb_device_handle *handle, int config)
 {
-	struct device_priv *dpriv = usbi_get_device_priv(handle->dev);
+	struct device_priv *dpriv = (struct device_priv *)handle->dev->os_priv;
 
 	if (dpriv->devname == NULL)
 		return (LIBUSB_ERROR_NOT_SUPPORTED);
 
-	usbi_dbg(HANDLE_CTX(handle), "bConfigurationValue %d", config);
+	usbi_dbg("bConfigurationValue %d", config);
 
 	if (ioctl(dpriv->fd, USB_SET_CONFIG, &config) < 0)
 		return _errno_to_libusb(errno);
@@ -329,13 +377,11 @@ obsd_set_configuration(struct libusb_device_handle *handle, int config)
 }
 
 int
-obsd_claim_interface(struct libusb_device_handle *handle, uint8_t iface)
+obsd_claim_interface(struct libusb_device_handle *handle, int iface)
 {
-	struct handle_priv *hpriv = usbi_get_device_handle_priv(handle);
+	struct handle_priv *hpriv = (struct handle_priv *)handle->os_priv;
 	int i;
 
-	UNUSED(iface);
-
 	for (i = 0; i < USB_MAX_ENDPOINTS; i++)
 		hpriv->endpoints[i] = -1;
 
@@ -343,13 +389,11 @@ obsd_claim_interface(struct libusb_device_handle *handle, uint8_t iface)
 }
 
 int
-obsd_release_interface(struct libusb_device_handle *handle, uint8_t iface)
+obsd_release_interface(struct libusb_device_handle *handle, int iface)
 {
-	struct handle_priv *hpriv = usbi_get_device_handle_priv(handle);
+	struct handle_priv *hpriv = (struct handle_priv *)handle->os_priv;
 	int i;
 
-	UNUSED(iface);
-
 	for (i = 0; i < USB_MAX_ENDPOINTS; i++)
 		if (hpriv->endpoints[i] >= 0)
 			close(hpriv->endpoints[i]);
@@ -358,16 +402,16 @@ obsd_release_interface(struct libusb_device_handle *handle, uint8_t iface)
 }
 
 int
-obsd_set_interface_altsetting(struct libusb_device_handle *handle, uint8_t iface,
-    uint8_t altsetting)
+obsd_set_interface_altsetting(struct libusb_device_handle *handle, int iface,
+    int altsetting)
 {
-	struct device_priv *dpriv = usbi_get_device_priv(handle->dev);
+	struct device_priv *dpriv = (struct device_priv *)handle->dev->os_priv;
 	struct usb_alt_interface intf;
 
 	if (dpriv->devname == NULL)
 		return (LIBUSB_ERROR_NOT_SUPPORTED);
 
-	usbi_dbg(HANDLE_CTX(handle), "iface %u, setting %u", iface, altsetting);
+	usbi_dbg("iface %d, setting %d", iface, altsetting);
 
 	memset(&intf, 0, sizeof(intf));
 
@@ -389,7 +433,7 @@ obsd_clear_halt(struct libusb_device_handle *handle, unsigned char endpoint)
 	if ((fd = _bus_open(handle->dev->bus_number)) < 0)
 		return _errno_to_libusb(errno);
 
-	usbi_dbg(HANDLE_CTX(handle), " ");
+	usbi_dbg("");
 
 	req.ucr_addr = handle->dev->device_address;
 	req.ucr_request.bmRequestType = UT_WRITE_ENDPOINT;
@@ -408,12 +452,20 @@ obsd_clear_halt(struct libusb_device_handle *handle, unsigned char endpoint)
 	return (LIBUSB_SUCCESS);
 }
 
+int
+obsd_reset_device(struct libusb_device_handle *handle)
+{
+	usbi_dbg("");
+
+	return (LIBUSB_ERROR_NOT_SUPPORTED);
+}
+
 void
 obsd_destroy_device(struct libusb_device *dev)
 {
-	struct device_priv *dpriv = usbi_get_device_priv(dev);
+	struct device_priv *dpriv = (struct device_priv *)dev->os_priv;
 
-	usbi_dbg(DEVICE_CTX(dev), " ");
+	usbi_dbg("");
 
 	free(dpriv->cdesc);
 	free(dpriv->devname);
@@ -423,11 +475,13 @@ int
 obsd_submit_transfer(struct usbi_transfer *itransfer)
 {
 	struct libusb_transfer *transfer;
+	struct handle_priv *hpriv;
 	int err = 0;
 
-	usbi_dbg(ITRANSFER_CTX(itransfer), " ");
+	usbi_dbg("");
 
 	transfer = USBI_TRANSFER_TO_LIBUSB_TRANSFER(itransfer);
+	hpriv = (struct handle_priv *)transfer->dev_handle->os_priv;
 
 	switch (transfer->type) {
 	case LIBUSB_TRANSFER_TYPE_CONTROL:
@@ -450,15 +504,13 @@ obsd_submit_transfer(struct usbi_transfer *itransfer)
 		}
 		err = _sync_gen_transfer(itransfer);
 		break;
-	case LIBUSB_TRANSFER_TYPE_BULK_STREAM:
-		err = LIBUSB_ERROR_NOT_SUPPORTED;
-		break;
 	}
 
 	if (err)
 		return (err);
 
-	usbi_signal_transfer_completion(itransfer);
+	if (write(hpriv->pipe[1], &itransfer, sizeof(itransfer)) < 0)
+		return _errno_to_libusb(errno);
 
 	return (LIBUSB_SUCCESS);
 }
@@ -466,23 +518,97 @@ obsd_submit_transfer(struct usbi_transfer *itransfer)
 int
 obsd_cancel_transfer(struct usbi_transfer *itransfer)
 {
-	UNUSED(itransfer);
-
-	usbi_dbg(ITRANSFER_CTX(itransfer), " ");
+	usbi_dbg("");
 
 	return (LIBUSB_ERROR_NOT_SUPPORTED);
 }
 
-int
-obsd_handle_transfer_completion(struct usbi_transfer *itransfer)
+void
+obsd_clear_transfer_priv(struct usbi_transfer *itransfer)
 {
-	return usbi_handle_transfer_completion(itransfer, LIBUSB_TRANSFER_COMPLETED);
+	usbi_dbg("");
+
+	/* Nothing to do */
+}
+
+int
+obsd_handle_events(struct libusb_context *ctx, struct pollfd *fds, nfds_t nfds,
+    int num_ready)
+{
+	struct libusb_device_handle *handle;
+	struct handle_priv *hpriv = NULL;
+	struct usbi_transfer *itransfer;
+	struct pollfd *pollfd;
+	int i, err = 0;
+
+	usbi_dbg("");
+
+	pthread_mutex_lock(&ctx->open_devs_lock);
+	for (i = 0; i < nfds && num_ready > 0; i++) {
+		pollfd = &fds[i];
+
+		if (!pollfd->revents)
+			continue;
+
+		hpriv = NULL;
+		num_ready--;
+		list_for_each_entry(handle, &ctx->open_devs, list,
+		    struct libusb_device_handle) {
+			hpriv = (struct handle_priv *)handle->os_priv;
+
+			if (hpriv->pipe[0] == pollfd->fd)
+				break;
+
+			hpriv = NULL;
+		}
+
+		if (NULL == hpriv) {
+			usbi_dbg("fd %d is not an event pipe!", pollfd->fd);
+			err = ENOENT;
+			break;
+		}
+
+		if (pollfd->revents & POLLERR) {
+			usbi_remove_pollfd(HANDLE_CTX(handle), hpriv->pipe[0]);
+			usbi_handle_disconnect(handle);
+			continue;
+		}
+
+		if (read(hpriv->pipe[0], &itransfer, sizeof(itransfer)) < 0) {
+			err = errno;
+			break;
+		}
+
+		if ((err = usbi_handle_transfer_completion(itransfer,
+		    LIBUSB_TRANSFER_COMPLETED)))
+			break;
+	}
+	pthread_mutex_unlock(&ctx->open_devs_lock);
+
+	if (err)
+		return _errno_to_libusb(err);
+
+	return (LIBUSB_SUCCESS);
+}
+
+int
+obsd_clock_gettime(int clkid, struct timespec *tp)
+{
+	usbi_dbg("clock %d", clkid);
+
+	if (clkid == USBI_CLOCK_REALTIME)
+		return clock_gettime(CLOCK_REALTIME, tp);
+
+	if (clkid == USBI_CLOCK_MONOTONIC)
+		return clock_gettime(CLOCK_MONOTONIC, tp);
+
+	return (LIBUSB_ERROR_INVALID_PARAM);
 }
 
 int
 _errno_to_libusb(int err)
 {
-	usbi_dbg(NULL, "error: %s (%d)", strerror(err), err);
+	usbi_dbg("error: %s (%d)", strerror(err), err);
 
 	switch (err) {
 	case EIO:
@@ -503,16 +629,16 @@ _errno_to_libusb(int err)
 int
 _cache_active_config_descriptor(struct libusb_device *dev)
 {
-	struct device_priv *dpriv = usbi_get_device_priv(dev);
+	struct device_priv *dpriv = (struct device_priv *)dev->os_priv;
 	struct usb_device_cdesc udc;
 	struct usb_device_fdesc udf;
-	void *buf;
+	unsigned char* buf;
 	int fd, len, err;
 
 	if ((fd = _bus_open(dev->bus_number)) < 0)
 		return _errno_to_libusb(errno);
 
-	usbi_dbg(DEVICE_CTX(dev), "fd %d, addr %d", fd, dev->device_address);
+	usbi_dbg("fd %d, addr %d", fd, dev->device_address);
 
 	udc.udc_bus = dev->bus_number;
 	udc.udc_addr = dev->device_address;
@@ -523,10 +649,10 @@ _cache_active_config_descriptor(struct libusb_device *dev)
 		return _errno_to_libusb(errno);
 	}
 
-	usbi_dbg(DEVICE_CTX(dev), "active bLength %d", udc.udc_desc.bLength);
+	usbi_dbg("active bLength %d", udc.udc_desc.bLength);
 
 	len = UGETW(udc.udc_desc.wTotalLength);
-	buf = malloc((size_t)len);
+	buf = malloc(len);
 	if (buf == NULL)
 		return (LIBUSB_ERROR_NO_MEM);
 
@@ -536,7 +662,7 @@ _cache_active_config_descriptor(struct libusb_device *dev)
 	udf.udf_size = len;
 	udf.udf_data = buf;
 
-	usbi_dbg(DEVICE_CTX(dev), "index %d, len %d", udf.udf_config_index, len);
+	usbi_dbg("index %d, len %d", udf.udf_config_index, len);
 
 	if (ioctl(fd, USB_DEVICE_GET_FDESC, &udf) < 0) {
 		err = errno;
@@ -562,10 +688,10 @@ _sync_control_transfer(struct usbi_transfer *itransfer)
 	struct usb_ctl_request req;
 
 	transfer = USBI_TRANSFER_TO_LIBUSB_TRANSFER(itransfer);
-	dpriv = usbi_get_device_priv(transfer->dev_handle->dev);
+	dpriv = (struct device_priv *)transfer->dev_handle->dev->os_priv;
 	setup = (struct libusb_control_setup *)transfer->buffer;
 
-	usbi_dbg(ITRANSFER_CTX(itransfer), "type 0x%x request 0x%x value 0x%x index %d length %d timeout %d",
+	usbi_dbg("type %x request %x value %x index %d length %d timeout %d",
 	    setup->bmRequestType, setup->bRequest,
 	    libusb_le16_to_cpu(setup->wValue),
 	    libusb_le16_to_cpu(setup->wIndex),
@@ -610,7 +736,7 @@ _sync_control_transfer(struct usbi_transfer *itransfer)
 
 	itransfer->transferred = req.ucr_actlen;
 
-	usbi_dbg(ITRANSFER_CTX(itransfer), "transferred %d", itransfer->transferred);
+	usbi_dbg("transferred %d", itransfer->transferred);
 
 	return (0);
 }
@@ -624,13 +750,13 @@ _access_endpoint(struct libusb_transfer *transfer)
 	int fd, endpt;
 	mode_t mode;
 
-	hpriv = usbi_get_device_handle_priv(transfer->dev_handle);
-	dpriv = usbi_get_device_priv(transfer->dev_handle->dev);
+	hpriv = (struct handle_priv *)transfer->dev_handle->os_priv;
+	dpriv = (struct device_priv *)transfer->dev_handle->dev->os_priv;
 
 	endpt = UE_GET_ADDR(transfer->endpoint);
 	mode = IS_XFERIN(transfer) ? O_RDONLY : O_WRONLY;
 
-	usbi_dbg(TRANSFER_CTX(transfer), "endpoint %d mode %d", endpt, mode);
+	usbi_dbg("endpoint %d mode %d", endpt, mode);
 
 	if (hpriv->endpoints[endpt] < 0) {
 		/* Pick the right endpoint node */
@@ -656,7 +782,7 @@ _sync_gen_transfer(struct usbi_transfer *itransfer)
 	int fd, nr = 1;
 
 	transfer = USBI_TRANSFER_TO_LIBUSB_TRANSFER(itransfer);
-	dpriv = usbi_get_device_priv(transfer->dev_handle->dev);
+	dpriv = (struct device_priv *)transfer->dev_handle->dev->os_priv;
 
 	if (dpriv->devname == NULL)
 		return (LIBUSB_ERROR_NOT_SUPPORTED);
